//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------

#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules

#include "midi_receive.h"
#include "tonetable.h"


#define  BUFFERSIZE  8

/* ------------------ */
/* - global value     */
/* ------------------ */

// receive data from MIDI in
BYTE  g_bRxReceive;

// receive ring buffer
BYTE  bRxBuffer[BUFFERSIZE];
// ring buffer pointer
BYTE  bReceivePointer, bProcessPointer;

// output trigger
BYTE  bOutputFlag;
BYTE  bOutputTone;

/* ------------------ */
/* - int handler      */
/* ------------------ */

/* --- RX8 receive intrrupt */
#pragma interrupt_handler ReceiveRx8
void ReceiveRx8()
{
	bRxBuffer[bReceivePointer] = g_bRxReceive;
	bReceivePointer++;
	bReceivePointer &= (BUFFERSIZE-1);
}


/* ------------------ */
/* - frances work     */
/* ------------------ */

/* --- PWM control */
void FrancesPWM()
{
	WORD  length;

	if ((bMidiOption & 0b00000001) != 0) {
		/* - SERVO control PWM */
		PWM16_1_WritePeriod(30052);
		length = 18 * bMidiMonoTone;
		length += 2087;
		PWM16_1_WritePulseWidth(length);
	}
	else {
		/* - tone start */
		if ((0 == bOutputFlag) && (1 == bMidiMonoTrigger)) {
			PWM16_1_Stop();
			PWM16_1_WritePeriod(ToneTable[bMidiMonoTone]);
			PWM16_1_WritePulseWidth(ToneTable[bMidiMonoTone]>>1);
			PWM16_1_Start();
			bOutputTone = bMidiMonoTone;
		}
		else if ((1 == bOutputFlag) && (bMidiMonoTone != bOutputTone)) {
			PWM16_1_Stop();
			PWM16_1_WritePeriod(ToneTable[bMidiMonoTone]);
			PWM16_1_WritePulseWidth(ToneTable[bMidiMonoTone]>>1);
			PWM16_1_Start();
			bOutputTone = bMidiMonoTone;
		}
		/* - tone stop */
		if ((bMidiOption & 0b00000010) == 0) {
			if ((1 == bOutputFlag) && (0 == bMidiMonoTrigger)) {
				PWM16_1_Stop();
				PWM16_1_WritePeriod(2);
				PWM16_1_WritePulseWidth(2);
				PWM16_1_Start();
			}
		}
	}
	bOutputFlag = bMidiMonoTrigger;
}

/* --- I/O control */
void FrancesOutput()
{
	BYTE monoout;
	
	monoout = bMidiMonoTone | (bMidiMonoTrigger <<7);
	PRT0DR = monoout;
	PRT2DR = bMidiOutPoly;

	FrancesPWM();
}

BYTE FrancesChannelRead()
{
	BYTE  p1;

	p1 = PRT1DR;
	if ((p1 & 0b00000001) == 1) {
		/* reset */
		MidiReset();
		PWM16_1_WritePeriod(2);
		PWM16_1_WritePulseWidth(2);\
	}
	p1 &= 0b00111100;
	p1 >>= 2;
//	p1=0;	
	return(p1);
}



/* ------------------ */
/* - main routine     */
/* ------------------ */

void main()
{
	BYTE  midiChannel, chsw;

	/* module */
	PWM8_2_WritePeriod(23);
	PWM8_2_WritePulseWidth(11);
	PWM16_1_WritePeriod(2);
	PWM16_1_WritePulseWidth(2);
	PWM8_1_Start();
	PWM8_2_Start();
	PWM16_1_Start();
	RX8_1_Start( RX8_1_PARITY_NONE );
	
	/* work initialize */
	bReceivePointer = bProcessPointer = 0;
	bOutputFlag = bOutputTone = 0;

	midiChannel = FrancesChannelRead();
	SetMidiChannel(midiChannel);
	MidiReset();

	/* int */
	RX8_1_EnableInt();
	M8C_EnableGInt;

	while(1) {
		while(bReceivePointer == bProcessPointer) {}

		MidiReceive(bRxBuffer[bProcessPointer]);

		/* channel change */
		chsw = FrancesChannelRead();
		if (chsw != midiChannel) {
			midiChannel = chsw;
			SetMidiChannel(midiChannel);
			MidiReset();
		}

		/* next */
		bProcessPointer++;
		bProcessPointer &= (BUFFERSIZE-1);
 	}
}
